---- Communicating game state to players

local umsg = umsg
local string = string
local table = table
local pairs = pairs

-- Basic status message to everyone
function GameMsg(msg)
   --PrintMessage(HUD_PRINTTALK, msg)

   umsg.Start("game_msg")
   umsg.String(msg)
   umsg.Bool(false)
   umsg.End()
end

-- Basic status message to single player or a recipientfilter
function PlayerMsg(ply_or_rfilter, msg, traitor_only)
   umsg.Start("game_msg", ply_or_rfilter)
   umsg.String(msg)
   umsg.Bool(traitor_only)
   umsg.End()
end

-- Traitor-specific message that will appear in a special color
function TraitorMsg(ply_or_rfilter, msg)
   PlayerMsg(ply_or_rfilter, msg, true)
end

-- Traitorchat
function TraitorChatMsg(msg)
   for k, v in pairs(player.GetAll()) do
      if ValidEntity(v) and v.is_traitor then
         v:ChatPrint(msg)
      end
   end
end

-- Round start info popup
function ShowRoundStartPopup()
   for k, v in pairs(player.GetAll()) do
      if ValidEntity(v) and v:Team() == TEAM_TERROR and v:Alive() then
         v:ConCommand("ttt_cl_startpopup")
      end
   end   
end

local function GetPlayerFilter(pred)
   local filter = RecipientFilter()
   for k, v in pairs(player.GetAll()) do
      if ValidEntity(v) and pred(v) then
         filter:AddPlayer(v)
      end
   end
   return filter
end

function GetTraitorFilter(alive_only)
   return GetPlayerFilter(function(p) return p:GetTraitor() and (not alive_only or p:Alive()) end)
end

function GetInnocentFilter(alive_only)
   return GetPlayerFilter(function(p) return not p:GetTraitor() and (not alive_only or p:Alive()) end)
end

---- Communication control
CreateConVar("ttt_limit_spectator_chat", "1", FCVAR_ARCHIVE + FCVAR_NOTIFY)
CreateConVar("ttt_limit_spectator_voice", "1", FCVAR_ARCHIVE + FCVAR_NOTIFY)

local mumbles = {"mumble", "mm", "hmm", "hum", "mum", "mbm", "mble", "ham", "mammaries", "political situation", "mrmm", "hrm", "uzbekistan", "mumu", "cheese export", "hmhm", "mmh", "mumble", "mphrrt", "mrh", "hmm", "mumble", "mbmm", "hmml", "mfrrm"}

-- While a round is active, spectators can only talk among themselves. When they
-- try to speak to all players they could divulge information about who killed
-- them. So we mumblify them. In detective mode, we shut them up entirely.
function GM:PlayerSay(ply, text, to_all)
   if not ValidEntity(ply) then return end

   if not to_all and ply:Team() != TEAM_SPEC and GetRoundState() == ROUND_ACTIVE then
      if ply.is_traitor then
         -- traitor chat handling
         TraitorChatMsg("(TRAITOR) ".. ply:Nick() .. "\t" .. text)
         return "" -- don't show as normal chat
      else
         ply:PrintMessage(HUD_PRINTTALK, "When innocent, use global chat to communicate.")
         return ""
      end
   end

   if to_all and GetRoundState() == ROUND_ACTIVE and ply:Team() == TEAM_SPEC then
      if DetectiveMode() then
         ply:PrintMessage(HUD_PRINTTALK, "While a round is in progress, spectators cannot speak to living players. Use team chat to speak with fellow spectators.")
         return ""
      end

      if not GetConVar("ttt_limit_spectator_chat"):GetBool() then
         return
      end

      local filtered = {}
      for k, v in pairs(string.Explode(" ", text)) do
         -- grab word characters and whitelisted interpunction
         -- necessary or leetspeek will be used (by trolls especially)
         local word, interp = string.match(v, "(%a*)([%.,;!%?]*)")
         if word != "" then
            table.insert(filtered, mumbles[math.random(1, #mumbles)] .. interp)
         end
      end

      -- make sure we have something to say
      if table.Count(filtered) < 1 then
         table.insert(filtered, mumbles[math.random(1, #mumbles)])
      end

      table.insert( filtered, 1, "[MUMBLED]")
      return table.concat(filtered, " ")
   end

   return text
end


-- Of course voice has to be limited as well
function GM:PlayerCanHearPlayersVoice( listener, speaker )
   if not ValidEntity(speaker) or not ValidEntity(listener) then
      return false
   end

   -- limited if specific convar is on, or we're in detective mode
   local limit = DetectiveMode() or GetConVar("ttt_limit_spectator_voice"):GetBool()
   if speaker:IsSpec() and not listener:IsSpec() and limit and GetRoundState() == ROUND_ACTIVE then
      return false
   elseif listener:IsSpec() and listener.mute_team == speaker:Team() then
      return false
   end

   -- traitors "team"chat by default
   if speaker:IsActiveTraitor() then
      return listener:IsActiveTraitor() or speaker.traitor_gvoice
   end

   return true
end

local function SendTraitorVoiceState(speaker, state)
   -- send umsg to living traitors that this is traitor-only talk
   local rf = GetTraitorFilter(true)
   
   -- tiny umsg in the hope of it arriving asap
   umsg.Start("tvo", rf)
   umsg.Short(speaker:EntIndex())
   umsg.Bool(state)
   umsg.End()
end


local function TraitorGlobalVoice(ply, cmd, args)
   if not ValidEntity(ply) or not ply:IsActiveTraitor() then return end
   if not #args == 1 then return end
   local state = tonumber(args[1])
   
   ply.traitor_gvoice = (state == 1)

   SendTraitorVoiceState(ply, ply.traitor_gvoice)
   MsgN("V: Received tvog " .. tostring(ply.traitor_gvoice) .. " from " .. tostring(ply))
end
concommand.Add("tvog", TraitorGlobalVoice)

local function MuteTeam(ply, cmd, args)
   if not ValidEntity(ply) or not ply:IsSpec() then return end
   if not #args == 1 and tonumber(args[1]) then return end

   ply.mute_team = tonumber(args[1])
end
concommand.Add("ttt_mute_team", MuteTeam)

local ttt_lastwords = CreateConVar("ttt_lastwords_chatprint", "0")

local LastWordContext = {
   [KILL_NORMAL] = "",
   [KILL_SUICIDE] = " *kills self*",
   [KILL_FALL] = " *SPLUT*",
   [KILL_BURN] = " *crackle*"
};

local function LastWords(ply, cmd, args)
   if ValidEntity(ply) and not ply:Alive() and #args > 1 then
      local id = args[1]
      if tonumber(id) == ply.last_words_id then
         local words = string.Trim(args[2])

         -- only append "--" if there's no ending interpunction
         local final = string.match(words, "[\.\!\?]$") != nil

         if ttt_lastwords:GetBool() or ply.death_type == KILL_FALL then
            -- add optional context relating to death type
            local context = LastWordContext[ply.death_type] or ""

            local text = "(LAST WORDS) " .. ply:Nick() .. "\t" .. words .. (final and "" or "--") .. context
            PrintMessage(HUD_PRINTTALK, text)
         end

         local rag = ply.server_ragdoll
         if ValidEntity(rag) and rag.player_ragdoll then
            rag.last_words = words
         end
      end
   end
end
concommand.Add("_deathsay", LastWords)



